// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef PRISON_VERSION_H
#define PRISON_VERSION_H

#define PRISON_VERSION_STRING "5.115.0"
#define PRISON_VERSION_MAJOR 5
#define PRISON_VERSION_MINOR 115
#define PRISON_VERSION_PATCH 0
#define PRISON_VERSION ((5<<16)|(115<<8)|(0))

#endif
